import java.io.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Scanner;
import java.util.zip.GZIPInputStream;

import edu.berkeley.nlp.lm.*;
import edu.berkeley.nlp.lm.collections.Iterators;
import edu.berkeley.nlp.lm.io.IOUtils;
import edu.berkeley.nlp.lm.io.LmReaders;
import edu.berkeley.nlp.lm.map.NgramMapWrapper;
import edu.berkeley.nlp.lm.util.LongRef;


import is2.data.SentenceData09;
import is2.transitionS2c.Parser;


/**
 * 
 */

/**
 *
 *
 */
public class Corrector {
	
	HashMap<String,HashMap<String,Integer>> google5grams;
	HashMap<String,HashMap<String,Integer>> google4grams;
	HashMap<String,HashMap<String,Integer>> google3grams;
	
	HashMap<String,HashMap<String,ArrayList<String>>> confSets;
	
	String fileNgrams;
	
	NgramLanguageModel<String>  lm;
	
	ArrayList<String> stopWords;
	
	private Parser tbparser = null;
	
	
	
		/**
	 * 
	 * @param fngrams
	 */
	public Corrector(String fngrams, String fconfSets, String languagemodel, String vocab, String fileswords, String depParserModel){
		
		/*System.out.println("Loading stopwords");
		stopWords=loadStopWords(fileswords);*/
		
		System.out.println("Loading dependency-parser-lemmatizer-tagger model");
		tbparser=new Parser(depParserModel); 
		
		System.out.println("Loading language model");
		lm = readBinary(true, vocab, languagemodel);
		
		
		String sentenceTest="me acabo de enterrar que hay un partido político que se llama ciudadanos";
		
		List<String> testList= Arrays.asList(sentenceTest.trim().split("\\s+"));
		float logProbCand=lm.scoreSentence(testList);
		System.out.println("Score of candidate sentence:"+sentenceTest+": "+ logProbCand);
		
		sentenceTest="me acabo de enterar que hay un partido político que se llama ciudadanos";
		
		List<String> testList2= Arrays.asList(sentenceTest.trim().split("\\s+"));
		float logProbCand2=lm.scoreSentence(testList2);
		System.out.println("Score of candidate sentence:"+sentenceTest+": "+ logProbCand2);
				
		System.out.println("Loaded");
		
		/*String s="92834y2783 723487238 kjsdfdjrfk12 174812734 yhhbhjbjh1";
		List<String> words = Arrays.asList(s.trim().split("\\s+"));
		System.out.println(lm.scoreSentence(words));
		
		String s1="akdajks asjkdak jasdjka jhaska ajskda";
		List<String> words1 = Arrays.asList(s1.trim().split("\\s+"));
		System.out.println(lm.scoreSentence(words1));
		
		String s2="hola esto es un frase de";
		List<String> words2 = Arrays.asList(s2.trim().split("\\s+"));
		System.out.println(lm.scoreSentence(words2));
		
		
		String s3="me acabo de enterara que";
		List<String> words3 = Arrays.asList(s3.trim().split("\\s+"));
		System.out.println(lm.scoreSentence(words3));
		
		
		String s4="me acabo de enterar que";
		List<String> words4 = Arrays.asList(s4.trim().split("\\s+"));
		System.out.println(lm.scoreSentence(words4));
		
		String s5="el sistema parece que funciona";
		List<String> words5= Arrays.asList(s5.trim().split("\\s+"));
		System.out.println(lm.scoreSentence(words5));
		
		String s6="una frase totalmente correcta";
		List<String> words6= Arrays.asList(s6.trim().split("\\s+"));
		System.out.println(lm.scoreSentence(words6));
		
		String s7="una frase toasdfa mal escrita";
		List<String> words7= Arrays.asList(s7.trim().split("\\s+"));
		System.out.println(lm.scoreSentence(words7));
		
		String s8="una tripa gorda";
		List<String> words8= Arrays.asList(s8.trim().split("\\s+"));
		System.out.println(lm.scoreSentence(words8));
		
		String s9="un tripa gorda";
		List<String> words9= Arrays.asList(s9.trim().split("\\s+"));
		System.out.println(lm.scoreSentence(words9));
		
		
		/*String s6="una frase totalmente correcta";
		List<String> words6= Arrays.asList(s6.trim().split("\\s+"));
		System.out.println(lm.scoreSentence(words6));
		
		String s6="una frase totalmente correcta";
		List<String> words6= Arrays.asList(s6.trim().split("\\s+"));
		System.out.println(lm.scoreSentence(words6));
		
		String s6="una frase totalmente correcta";
		List<String> words6= Arrays.asList(s6.trim().split("\\s+"));
		System.out.println(lm.scoreSentence(words6));*/
		
			
		
		
		fileNgrams=fngrams;
		
		google3grams=new HashMap<String,HashMap<String,Integer>>();
		google4grams=new HashMap<String,HashMap<String,Integer>>();
		google5grams=new HashMap<String,HashMap<String,Integer>>();
		
		confSets=new HashMap<String,HashMap<String,ArrayList<String>>>();

		System.out.println("Loading confusion sets...");
		loadConfSets(fconfSets);
		System.out.println("Loaded.");		
		
		System.out.println("Loading google ngrams...");
		loadNgrams(fngrams);
		System.out.println("Loaded.");
		
	}
		
	private ArrayList<String> loadStopWords(String fileswords) {
			// TODO Auto-generated method stub
		
			ArrayList<String> out=new ArrayList<String>();
			try {
				BufferedReader br=new BufferedReader(new FileReader(fileswords));
				while(br.ready()){
					String line=br.readLine();
					if (!out.contains(line)){
						out.add(line);
					}
				}
				br.close();
				
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return out;
		}

	private void loadConfSets(String fconfSets) {
		// TODO Auto-generated method stub
		try {
			BufferedReader br=new BufferedReader(new FileReader(fconfSets));
			String word="";
			ArrayList<String> confSet=new ArrayList<String>();
			while(br.ready()){
				String line=br.readLine();
				if (line.startsWith("Word")){
					String[] split=line.split(":");
					word=split[1];
					confSet=new ArrayList<String>();
				}
				if (line.startsWith("Lev: 1:")){
					String list=line.replaceAll("Lev: 1:\\[","");
					list=list.replaceAll("\\]","");
					String[] spList=list.split(", ");
					for(int i=0;i<spList.length;i++){
						if (!spList[i].isEmpty()) confSet.add(spList[i]);
					}
				}
				if (line.startsWith("Lev: 2:")){
					String list=line.replaceAll("Lev: 2:\\[","");						
					list=list.replaceAll("\\]","");
					String[] spList=list.split(", ");
					for(int i=0;i<spList.length;i++){
						if (!spList[i].isEmpty()) confSet.add(spList[i]);
					}
				}
				if (line.startsWith("Anagramas")){
					String list=line.replaceAll("Anagramas:\\[","");						
					list=list.replaceAll("\\]","");
					String[] spList=list.split(", ");
					for(int i=0;i<spList.length;i++){
						if (!spList[i].isEmpty()) confSet.add(spList[i]);
					}
						
				}
				if (line.startsWith("Combined")){
					/*String list=line.replaceAll("Combined:\\[","");						
					list=list.replaceAll("\\]","");
					String[] spList=list.split(", ");
					for(int i=0;i<spList.length;i++){
						confSet.add(spList[i]);
					}*/
				}
				if (line.isEmpty() && !word.isEmpty()){
					//store the data in memory
					HashMap<String,ArrayList<String>> hashList=confSets.get(word.length()+word.substring(0, 1));
					if (hashList==null) hashList=new HashMap<String,ArrayList<String>>();
					ArrayList<String> set=hashList.get(word);
					if (set==null) set=new ArrayList<String>();
					set.addAll(confSet);
					hashList.put(word,set);
					confSets.put(word.length()+word.substring(0, 1),hashList);
					
					
				}
			}
			//System.out.println(confSets);
			br.close();
			
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void loadNgrams(String fngrams) {
		// TODO Auto-generated method stub

		File folder=new File(fngrams);
		File[] files=folder.listFiles();
		for (int f=0;f<files.length;f++){
//		for (int f=0;f<1;f++){
			try {	
				InputStream fileStream = new FileInputStream(files[f].toString());
				InputStream gzipStream = new GZIPInputStream(fileStream);
				Reader decoder = new InputStreamReader(gzipStream, "UTF8");
				BufferedReader buffered = new BufferedReader(decoder);
				while(buffered.ready()){
					String line=buffered.readLine();
					
					String[] lineSplit=line.split("\t");
					if (line.contains("_") && !line.contains("_END_")){
						//System.out.println(line);
						String fivegram=lineSplit[0];
						
						String procNgram="";
						String[] fs=fivegram.split(" ");
						for(int i=0;i<fs.length;i++){
							if (fs[i].contains("_") && !fs[i].isEmpty()){
								String[] split=fs[i].split("_");
								//System.out.println(fs[i]);
								if (split!=null && split.length>0)
									fs[i]=split[0];
								//System.out.println(fs[i]);
							}
							fs[i]=fs[i].toLowerCase();
							procNgram+=fs[i]+" ";
						}
						procNgram=procNgram.substring(0, procNgram.length()-1);
						Integer frequency=Integer.parseInt(lineSplit[3]);
				
					//3 grams
							String procSplit[]=procNgram.split(" ");
							ArrayList<String> trigrams=new ArrayList<String>();
							if (procSplit.length>=3)
								trigrams.add(procSplit[0]+" "+procSplit[1]+" "+procSplit[2]);
							if (procSplit.length>=4)
								trigrams.add(procSplit[1]+" "+procSplit[2]+" "+procSplit[3]);
							if (procSplit.length>=5)
								trigrams.add(procSplit[2]+" "+procSplit[3]+" "+procSplit[4]);
						
							Iterator<String> it3=trigrams.iterator();
							while(it3.hasNext()){
								String ngr=it3.next();
								this.load3grams(ngr,frequency);
							}

					//4 grams
							//procSplit[]=procNgram.split(" ");
							ArrayList<String> fourgrams=new ArrayList<String>();
							if (procSplit.length>=4)
								fourgrams.add(procSplit[0]+" "+procSplit[1]+" "+procSplit[2]+" "+procSplit[3]);
							if (procSplit.length>=5)
								fourgrams.add(procSplit[1]+" "+procSplit[2]+" "+procSplit[3]+" "+procSplit[4]);
							Iterator<String> it4=fourgrams.iterator();
							while(it4.hasNext()){
								String ngr=it4.next();
								this.load4grams(ngr,frequency);
							}

					// 5 grams
							this.load5grams(procNgram,frequency);
							//System.out.println(out);

					}
				}
				buffered.close();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
	}

	private void load3grams(String ngr,Integer frequency){
		
		String firstKey=ngr.substring(0, 2);
		HashMap<String,Integer> hash3=this.google3grams.get(firstKey);
		if (hash3==null) hash3=new HashMap<String,Integer>();
		Integer value=hash3.get(ngr);
		if (value==null) value=0;
		value+=frequency;
		hash3.put(ngr, value);
		google3grams.put(firstKey,hash3);
		
		
	}
	
	private void load4grams(String ngr,Integer frequency){
		String firstKey=ngr.substring(0, 2);
		HashMap<String,Integer> hash4=this.google4grams.get(firstKey);
		if (hash4==null) hash4=new HashMap<String,Integer>();
		Integer value=hash4.get(ngr);
		if (value==null) value=0;
		value+=frequency;
		hash4.put(ngr, value);
		google4grams.put(firstKey,hash4);

	}

	private void load5grams(String ngr, Integer frequency){
		String firstKey=ngr.substring(0, 2);
		HashMap<String,Integer> hash5=this.google5grams.get(firstKey);
		if (hash5==null) hash5=new HashMap<String,Integer>();
		Integer value=hash5.get(ngr);
		if (value==null) value=0;
		value+=frequency;
		hash5.put(ngr, value);
		google5grams.put(firstKey,hash5);

	}

	
	
	/*public void run(String ngram){
		ngram=ngram.toLowerCase();
		

		String[] split=ngram.split(" ");
		String firstChars=ngram.substring(0,2);  //El fichero es siempre el que inicia la frase.
		
		ArrayList<String> trigrams=new ArrayList<String>();
		trigrams.add(split[0]+" "+split[1]+" "+split[2]);
		trigrams.add(split[1]+" "+split[2]+" "+split[3]);
		trigrams.add(split[2]+" "+split[3]+" "+split[4]);
		
		ArrayList<String> fourgrams=new ArrayList<String>();
		fourgrams.add(split[0]+" "+split[1]+" "+split[2]+" "+split[3]);
		fourgrams.add(split[1]+" "+split[2]+" "+split[3]+" "+split[4]);
		
		
		System.out.println("------------------------------");
		System.out.println("Sentence:"+ngram);
		System.out.println();
		
		
		Iterator<String> it3=trigrams.iterator();
		while(it3.hasNext()){
			String ngr=it3.next();
			System.out.println(getFrequencyOfNgram(ngr,3,firstChars));
		}
		
		Iterator<String> it4=fourgrams.iterator();
		while(it4.hasNext()){
			String ngr=it4.next();
			System.out.println(getFrequencyOfNgram(ngr,4,firstChars));
		}
		System.out.println(getFrequencyOfNgram(ngram,5,firstChars));
		
		System.out.println("------------------------------");
		
		
	}*/
	
	/**
	 * 
	 * @param ngram
	 * @param n can be 3, 4 or 5
	 * @return
	 */
	private int getFrequencyOfNgram(String ngram, int n, String file){
		Integer out=0;
		//System.out.println(ngram);
		try {
			InputStream fileStream = new FileInputStream(this.fileNgrams+"googlebooks-spa-all-5gram-20120701-"+file+".gz");
			InputStream gzipStream = new GZIPInputStream(fileStream);
			Reader decoder = new InputStreamReader(gzipStream, "UTF8");
			BufferedReader buffered = new BufferedReader(decoder);
			while(buffered.ready()){
				String line=buffered.readLine();
				
				String[] lineSplit=line.split("\t");
				if (line.contains("_") && !line.contains("_END_")){
					//System.out.println(line);
					String fivegram=lineSplit[0];
					
					String procNgram="";
					String[] fs=fivegram.split(" ");
					for(int i=0;i<5;i++){
						if (fs[i].contains("_") && !fs[i].isEmpty()){
							String[] split=fs[i].split("_");
							//System.out.println(fs[i]);
							fs[i]=split[0];
							//System.out.println(fs[i]);
						}
						fs[i]=fs[i].toLowerCase();
						procNgram+=fs[i]+" ";
					}
					procNgram=procNgram.substring(0, procNgram.length()-1);
					if (n==3){
						String procSplit[]=procNgram.split(" ");
						ArrayList<String> trigrams=new ArrayList<String>();
						trigrams.add(procSplit[0]+" "+procSplit[1]+" "+procSplit[2]);
						trigrams.add(procSplit[1]+" "+procSplit[2]+" "+procSplit[3]);
						trigrams.add(procSplit[2]+" "+procSplit[3]+" "+procSplit[4]);
					
				
					
						Iterator<String> it3=trigrams.iterator();
						while(it3.hasNext()){
							String ngr=it3.next();
							if (ngr.equals(ngram)){
								Integer frequency=Integer.parseInt(lineSplit[3]);
								out+=frequency;
							}
						}

					}else if (n==4){
						String procSplit[]=procNgram.split(" ");
						ArrayList<String> fourgrams=new ArrayList<String>();
						fourgrams.add(procSplit[0]+" "+procSplit[1]+" "+procSplit[2]+" "+procSplit[3]);
						fourgrams.add(procSplit[1]+" "+procSplit[2]+" "+procSplit[3]+" "+procSplit[4]);
						Iterator<String> it4=fourgrams.iterator();
						while(it4.hasNext()){
							String ngr=it4.next();
							if (ngr.equals(ngram)){
								Integer frequency=Integer.parseInt(lineSplit[3]);
								out+=frequency;
							}
						}
						
					}
					else if (procNgram.equals(ngram)){
						Integer frequency=Integer.parseInt(lineSplit[3]);
						out+=frequency;
						//System.out.println(out);
					}
				}
				
				
			}
			buffered.close();
			
		} catch (Exception e) {
			// TODO Auto-generated catch block
			//e.printStackTrace();
			return out.intValue();
		}
		return out.intValue();
	}
	
	/**
	 * 
	 * @param ngram
	 * @param n
	 * @return
	 */
	private int getFrequencyOfNgram1stWord(String ngram, int n){
		Integer out=0;
		//System.out.println(ngram);
		if (n==3){
			String procSplit[]=ngram.split(" ");
			ArrayList<String> trigrams=new ArrayList<String>();
			trigrams.add(procSplit[0]+" "+procSplit[1]+" "+procSplit[2]);
			//trigrams.add(procSplit[1]+" "+procSplit[2]+" "+procSplit[3]);
			//trigrams.add(procSplit[2]+" "+procSplit[3]+" "+procSplit[4]);
					
				
			Iterator<String> it3=trigrams.iterator();
			while(it3.hasNext()){
				String ngr=it3.next();
				HashMap<String,Integer> hash3rd=this.google3grams.get(ngr.substring(0, 2));
				if (hash3rd!=null){
					Integer freq3rd=hash3rd.get(ngr);
					if (freq3rd!=null) out+=freq3rd;
				}
			}
		}else if (n==4){
			String procSplit[]=ngram.split(" ");
			ArrayList<String> fourgrams=new ArrayList<String>();
			//if (!stopWords.contains(procSplit[3])){
				fourgrams.add(procSplit[0]+" "+procSplit[1]+" "+procSplit[2]+" "+procSplit[3]);
			//}
			//fourgrams.add(procSplit[1]+" "+procSplit[2]+" "+procSplit[3]+" "+procSplit[4]);
			Iterator<String> it4=fourgrams.iterator();
			while(it4.hasNext()){
				String ngr=it4.next();
				HashMap<String,Integer> hash4th=this.google4grams.get(ngr.substring(0, 2));
				if (hash4th!=null){
					Integer freq4th=hash4th.get(ngr);
					if (freq4th!=null) out+=freq4th;
				}
			}
						
		}
		else {
			HashMap<String,Integer> hash5th=this.google4grams.get(ngram.substring(0, 2));
			if (hash5th!=null){
				Integer freq5th=hash5th.get(ngram);
				if (freq5th!=null) out+=freq5th;
			}
		}		
		return out.intValue();

	}
	
	
	/**
	 * 
	 * @param ngram
	 * @param n
	 * @return
	 */
	private int getFrequencyOfNgram3rdWord(String ngram, int n){
		Integer out=0;
		//System.out.println(ngram);
		if (n==3){
			String procSplit[]=ngram.split(" ");
			ArrayList<String> trigrams=new ArrayList<String>();
			trigrams.add(procSplit[0]+" "+procSplit[1]+" "+procSplit[2]);
			trigrams.add(procSplit[1]+" "+procSplit[2]+" "+procSplit[3]);
			trigrams.add(procSplit[2]+" "+procSplit[3]+" "+procSplit[4]);
					
				
			Iterator<String> it3=trigrams.iterator();
			while(it3.hasNext()){
				String ngr=it3.next();
				HashMap<String,Integer> hash3rd=this.google3grams.get(ngr.substring(0, 2));
				if (hash3rd!=null){
					Integer freq3rd=hash3rd.get(ngr);
					if (freq3rd!=null) out+=freq3rd;
				}
			}
		}else if (n==4){
			String procSplit[]=ngram.split(" ");
			ArrayList<String> fourgrams=new ArrayList<String>();
			//if (!stopWords.contains(procSplit[3])){
				fourgrams.add(procSplit[0]+" "+procSplit[1]+" "+procSplit[2]+" "+procSplit[3]);
			//}
			//if (!stopWords.contains(procSplit[4])){
				fourgrams.add(procSplit[1]+" "+procSplit[2]+" "+procSplit[3]+" "+procSplit[4]);
			//}
			Iterator<String> it4=fourgrams.iterator();
			while(it4.hasNext()){
				String ngr=it4.next();
				HashMap<String,Integer> hash4th=this.google4grams.get(ngr.substring(0, 2));
				if (hash4th!=null){
					Integer freq4th=hash4th.get(ngr);
					if (freq4th!=null) out+=freq4th;
				}
			}
						
		}
		else {
			HashMap<String,Integer> hash5th=this.google4grams.get(ngram.substring(0, 2));
			if (hash5th!=null){
				Integer freq5th=hash5th.get(ngram);
				if (freq5th!=null) out+=freq5th;
			}
		}		
		return out.intValue();

	}
	
	/**
	 * 
	 * @param ngram
	 * @param n
	 * @return
	 */
	private int getFrequencyOfNgram2ndWord(String ngram, int n){
		Integer out=0;
		//System.out.println(ngram);
		if (n==3){
			String procSplit[]=ngram.split(" ");
			ArrayList<String> trigrams=new ArrayList<String>();
			trigrams.add(procSplit[0]+" "+procSplit[1]+" "+procSplit[2]);
			trigrams.add(procSplit[1]+" "+procSplit[2]+" "+procSplit[3]);
			//trigrams.add(procSplit[2]+" "+procSplit[3]+" "+procSplit[4]);
					
				
			Iterator<String> it3=trigrams.iterator();
			while(it3.hasNext()){
				String ngr=it3.next();
				HashMap<String,Integer> hash3rd=this.google3grams.get(ngr.substring(0, 2));
				if (hash3rd!=null){
					Integer freq3rd=hash3rd.get(ngr);
					if (freq3rd!=null) out+=freq3rd;
				}
			}
		}else if (n==4){
			String procSplit[]=ngram.split(" ");
			ArrayList<String> fourgrams=new ArrayList<String>();
			//if (!stopWords.contains(procSplit[3])){
				fourgrams.add(procSplit[0]+" "+procSplit[1]+" "+procSplit[2]+" "+procSplit[3]);
			//}
			//if (!stopWords.contains(procSplit[4])){
				fourgrams.add(procSplit[1]+" "+procSplit[2]+" "+procSplit[3]+" "+procSplit[4]);
			//}
			Iterator<String> it4=fourgrams.iterator();
			while(it4.hasNext()){
				String ngr=it4.next();
				HashMap<String,Integer> hash4th=this.google4grams.get(ngr.substring(0, 2));
				if (hash4th!=null){
					Integer freq4th=hash4th.get(ngr);
					if (freq4th!=null) out+=freq4th;
				}
			}
						
		}
		else {
			HashMap<String,Integer> hash5th=this.google4grams.get(ngram.substring(0, 2));
			if (hash5th!=null){
				Integer freq5th=hash5th.get(ngram);
				if (freq5th!=null) out+=freq5th;
			}
		}		
		return out.intValue();

	}
	
	/**
	 * 
	 * @param ngram
	 * @param n
	 * @return
	 */
	private int getFrequencyOfNgram4thWord(String ngram, int n){
		Integer out=0;
		//System.out.println(ngram);
		if (n==3){
			String procSplit[]=ngram.split(" ");
			ArrayList<String> trigrams=new ArrayList<String>();
			//trigrams.add(procSplit[0]+" "+procSplit[1]+" "+procSplit[2]);
			trigrams.add(procSplit[1]+" "+procSplit[2]+" "+procSplit[3]);
			trigrams.add(procSplit[2]+" "+procSplit[3]+" "+procSplit[4]);
					
				
			Iterator<String> it3=trigrams.iterator();
			while(it3.hasNext()){
				String ngr=it3.next();
				HashMap<String,Integer> hash3rd=this.google3grams.get(ngr.substring(0, 2));
				if (hash3rd!=null){
					Integer freq3rd=hash3rd.get(ngr);
					if (freq3rd!=null) out+=freq3rd;
				}
			}
		}else if (n==4){
			String procSplit[]=ngram.split(" ");
			ArrayList<String> fourgrams=new ArrayList<String>();
			//if (!stopWords.contains(procSplit[3])){
				fourgrams.add(procSplit[0]+" "+procSplit[1]+" "+procSplit[2]+" "+procSplit[3]);
			//}
			//if (!stopWords.contains(procSplit[4])){
				fourgrams.add(procSplit[1]+" "+procSplit[2]+" "+procSplit[3]+" "+procSplit[4]);
			//}
			Iterator<String> it4=fourgrams.iterator();
			while(it4.hasNext()){
				String ngr=it4.next();
				HashMap<String,Integer> hash4th=this.google4grams.get(ngr.substring(0, 2));
				if (hash4th!=null){
					Integer freq4th=hash4th.get(ngr);
					if (freq4th!=null) out+=freq4th;
				}
			}		
		}
		else {
			HashMap<String,Integer> hash5th=this.google4grams.get(ngram.substring(0, 2));
			if (hash5th!=null){
				Integer freq5th=hash5th.get(ngram);
				if (freq5th!=null) out+=freq5th;
			}
		}		
		return out.intValue();
	}
	
	
	private int getFrequencyOfNgram5thWord(String ngram, int n){
		Integer out=0;
		//System.out.println(ngram);
		if (n==3){
			String procSplit[]=ngram.split(" ");
			ArrayList<String> trigrams=new ArrayList<String>();
			//trigrams.add(procSplit[0]+" "+procSplit[1]+" "+procSplit[2]);
			//trigrams.add(procSplit[1]+" "+procSplit[2]+" "+procSplit[3]);
			trigrams.add(procSplit[2]+" "+procSplit[3]+" "+procSplit[4]);
					
				
			Iterator<String> it3=trigrams.iterator();
			while(it3.hasNext()){
				String ngr=it3.next();
				HashMap<String,Integer> hash3rd=this.google3grams.get(ngr.substring(0, 2));
				if (hash3rd!=null){
					Integer freq3rd=hash3rd.get(ngr);
					if (freq3rd!=null) out+=freq3rd;
				}
			}
		}else if (n==4){
			String procSplit[]=ngram.split(" ");
			ArrayList<String> fourgrams=new ArrayList<String>();
			//fourgrams.add(procSplit[0]+" "+procSplit[1]+" "+procSplit[2]+" "+procSplit[3]);
			fourgrams.add(procSplit[1]+" "+procSplit[2]+" "+procSplit[3]+" "+procSplit[4]);
			Iterator<String> it4=fourgrams.iterator();
			while(it4.hasNext()){
				String ngr=it4.next();
				HashMap<String,Integer> hash4th=this.google4grams.get(ngr.substring(0, 2));
				if (hash4th!=null){
					Integer freq4th=hash4th.get(ngr);
					if (freq4th!=null) out+=freq4th;
				}
			}
						
		}
		else {
			HashMap<String,Integer> hash5th=this.google4grams.get(ngram.substring(0, 2));
			if (hash5th!=null){
				Integer freq5th=hash5th.get(ngram);
				if (freq5th!=null) out+=freq5th;
			}
		}		
		return out.intValue();

	}

	
	
	private ArrayList<String> FifthWord(String ngram) {
		// TODO Auto-generated method stub
	
		ArrayList<String> list=new ArrayList<String>();
		
		String split[]=ngram.split(" ");
		if (split.length>4){
			ArrayList<String> confSet=getConfSet(split[4]);
			if (confSet!=null) {
				Iterator<String> it=confSet.iterator();
				while(it.hasNext()){
					list.add(it.next());
			
				}
			}
		}
		return list;
	}



	private ArrayList<String> FourthWord(String ngram) {
		// TODO Auto-generated method stub
		ArrayList<String> list=new ArrayList<String>();
		
		String split[]=ngram.split(" ");
		if (split.length>3){
			ArrayList<String> confSet=getConfSet(split[3]);
			if (confSet!=null) {
				Iterator<String> it=confSet.iterator();
				while(it.hasNext()){
					list.add(it.next());
			
				}
			}
		}
		return list;
	}



	private ArrayList<String> ThirdWord(String ngram) {
		// TODO Auto-generated method stub
		ArrayList<String> list=new ArrayList<String>();
		
		String split[]=ngram.split(" ");
		if (split.length>2){
			ArrayList<String> confSet=getConfSet(split[2]);
			if (confSet!=null) {
				Iterator<String> it=confSet.iterator();
				while(it.hasNext()){
					list.add(it.next());
			
				}
			}
		}
		return list;
	}



	private ArrayList<String> SecondWord(String ngram) {
		// TODO Auto-generated method stub
		ArrayList<String> list=new ArrayList<String>();
		
		String split[]=ngram.split(" ");
		if (split.length>1){
			ArrayList<String> confSet=getConfSet(split[1]);
			if (confSet!=null) {
				Iterator<String> it=confSet.iterator();
				while(it.hasNext()){
					list.add(it.next());
			
				}
			}
		}
		return list;
	}


	private ArrayList<String> FirstWord(String ngram) {
		// TODO Auto-generated method stub
		ArrayList<String> list=new ArrayList<String>();
		
		String split[]=ngram.split(" ");
		if (split.length>0){
			ArrayList<String> confSet=getConfSet(split[0]);
			if (confSet!=null) {
				Iterator<String> it=confSet.iterator();
				while(it.hasNext()){
					list.add(it.next());
			
				}
			}
		}
		return list;
	}
	
	
	/**
	 * This method returns the confusion set for a word
	 * @param word
	 * @return
	 */
	private ArrayList<String> getConfSet(String word){
		try{
			//System.out.println(this.confSets.get(word.length()+word.substring(0, 1)).get(word));
		return this.confSets.get(word.length()+word.substring(0, 1)).get(word);
		}catch(Exception e){
			return new ArrayList<String>();
		}
	}



	/**
	 * 
	 * @param args 0-> google ngrams, 1->confsets, 2->test set.
	 */
	public static void main (String[] args){
		
		
		Corrector c=new Corrector(args[0],args[1], args[3],args[4],args[5],args[6]); //args2=langmodel, args3=vocab
		
		//c.logProb("hola esto es una frase de");
		
		
		BufferedReader br;
		try {
			br = new BufferedReader(new FileReader(args[2]));
			while(br.ready()){
				String line=br.readLine();
				c.runCorrector(line);
				
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		
			
		/*try {
			BufferedReader br = new BufferedReader(new FileReader(args[2]));
			HashMap<Integer,ArrayList<String>> suggestions=new HashMap<Integer,ArrayList<String>>();
			while(br.ready()){
				
				long time_start, time_end;
				time_start = System.currentTimeMillis();

				String line=br.readLine(); //asumimos que está tokenizado...
				System.out.println("---------------------------------------------------");
				line=line.toLowerCase();
				System.out.println(line);
				
				
				ArrayList<String> fivegrams=new ArrayList<String>();
				
				
				String split[]=line.split(" ");
				
				//word0 word1 word2 word3 word4 word5 word6 word7...
				//HashMap<Integer,ArrayList<String>> suggestions=new HashMap<Integer,ArrayList<String>>();
				
				
				//dividirlo en ngrams de 5.
				for(int i=0;i<split.length-4;i++){
					String fivegram="";
					for(int j=i;j<5+i&&j<split.length;j++){
						fivegram+=split[j]+" ";
					}
					fivegram=fivegram.substring(0,fivegram.length()-1);
					if (!fivegrams.contains(fivegram)) fivegrams.add(fivegram);
				}
				System.out.println("f:"+fivegrams);
				
				Iterator<String> itfive=fivegrams.iterator();
				int ngramCount=0;
				while(itfive.hasNext()){
					String ngram=itfive.next();
					ArrayList<String> firstWordConfList=c.FirstWord(ngram);
					ArrayList<String> secondWordConfList=c.SecondWord(ngram);
					ArrayList<String> thirdWordConfList=c.ThirdWord(ngram);
					ArrayList<String> fourthWordConfList=c.FourthWord(ngram);
					ArrayList<String> fifthWordConfList=c.FifthWord(ngram);
					
					
					String splitNgram[]=ngram.split(" ");
					
					//System.out.println(ngram);
					System.out.println("Finding candidates for:"+splitNgram[0]);
					
					int frequencyOfNgram1st=c.getFrequencyOfNgram1stWord(ngram, 5)+c.getFrequencyOfNgram1stWord(ngram, 4)+c.getFrequencyOfNgram1stWord(ngram, 3);
					System.out.println(" Freq of '"+splitNgram[0]+"' in this context:"+frequencyOfNgram1st);
					
					Iterator<String> it1=firstWordConfList.iterator();
					while(it1.hasNext()){
						String word=it1.next();
						String testNgram=word+" "+splitNgram[1]+" "+splitNgram[2]+" "+splitNgram[3]+" "+splitNgram[4];
						System.out.print("\tTEST:"+testNgram+":");
						
						int freq=c.getFrequencyOfNgram1stWord(testNgram, 5)+c.getFrequencyOfNgram1stWord(testNgram, 4)+c.getFrequencyOfNgram1stWord(testNgram, 3);
						System.out.println(freq);
						//comprobar si freq es mayor que frequencyOfNgram1st
						if (freq>frequencyOfNgram1st){
							System.out.println("\t"+word +" seems to be a better word because its frequency is:"+freq);
							ArrayList<String> sug=suggestions.get(new Integer(ngramCount));
							if (sug==null) sug=new ArrayList<String>();
							if (!sug.contains(word)) sug.add(word);
							suggestions.put(new Integer(ngramCount), sug);
							
							
						}
					}
					System.out.println("Finding candidates for:"+splitNgram[1]);
					
					int frequencyOfNgram2nd=c.getFrequencyOfNgram2ndWord(ngram, 5)+c.getFrequencyOfNgram2ndWord(ngram, 4)+c.getFrequencyOfNgram2ndWord(ngram, 3);
					System.out.println(" Freq of '"+splitNgram[1]+"' in this context:"+frequencyOfNgram2nd);
					
					Iterator<String> it2=secondWordConfList.iterator();
					while(it2.hasNext()){
						String word=it2.next();
						String testNgram=split[0]+" "+word+" "+splitNgram[2]+" "+splitNgram[3]+" "+splitNgram[4];
						System.out.print("\tTEST:"+testNgram+":");
						
						int freq=c.getFrequencyOfNgram2ndWord(testNgram, 5)+c.getFrequencyOfNgram2ndWord(testNgram, 4)+c.getFrequencyOfNgram2ndWord(testNgram, 3);
						System.out.println(freq);
						//comprobar si freq es mayor que frequencyOfNgram2nd
						if (freq>frequencyOfNgram2nd){
							System.out.println("\t"+word +" seems to be a better word because its frequency is:"+freq);				
							ArrayList<String> sug=suggestions.get(new Integer(ngramCount+1));
							if (sug==null) sug=new ArrayList<String>();
							if (!sug.contains(word)) sug.add(word);
							suggestions.put(new Integer(ngramCount+1), sug);
						}
						
					}
					
					System.out.println("Finding candidates for:"+splitNgram[2]);
					
					int frequencyOfNgram3rd=c.getFrequencyOfNgram3rdWord(ngram, 5)+c.getFrequencyOfNgram3rdWord(ngram, 4)+c.getFrequencyOfNgram3rdWord(ngram, 3);
					System.out.println(" Freq of '"+splitNgram[2]+"' in this context:"+frequencyOfNgram3rd);
					
					Iterator<String> it3=thirdWordConfList.iterator();
					while(it3.hasNext()){
						String word=it3.next();
						String testNgram=splitNgram[0]+" "+splitNgram[1]+" "+word+" "+splitNgram[3]+" "+splitNgram[4];
						System.out.print("\tTEST:"+testNgram+":");
						
						int freq=c.getFrequencyOfNgram3rdWord(testNgram, 5)+c.getFrequencyOfNgram3rdWord(testNgram, 4)+c.getFrequencyOfNgram3rdWord(testNgram, 3);
						System.out.println(freq);
						if (freq>frequencyOfNgram3rd){
							System.out.println("\t"+word +" seems to be a better word because its frequency is:"+freq);
							ArrayList<String> sug=suggestions.get(new Integer(ngramCount+2));
							if (sug==null) sug=new ArrayList<String>();
							if (!sug.contains(word)) sug.add(word);
							suggestions.put(new Integer(ngramCount+2), sug);
						}
						//comprobar si freq es mayor que frequencyOfNgram3rd
					}
					
					System.out.println("Finding candidates for:"+splitNgram[3]);
					
					int frequencyOfNgram4th=c.getFrequencyOfNgram4thWord(ngram, 5)+c.getFrequencyOfNgram4thWord(ngram, 4)+c.getFrequencyOfNgram4thWord(ngram, 3);
					System.out.println(" Freq of '"+splitNgram[3]+"' in this context:"+frequencyOfNgram4th);
					
					Iterator<String> it4=fourthWordConfList.iterator();
					while(it4.hasNext()){
						String word=it4.next();
						String testNgram=splitNgram[0]+" "+splitNgram[1]+" "+splitNgram[2]+" "+word+" "+splitNgram[4];
						System.out.print("\tTEST:"+testNgram+":");
						
						int freq=c.getFrequencyOfNgram4thWord(testNgram, 5)+c.getFrequencyOfNgram4thWord(testNgram, 4)+c.getFrequencyOfNgram4thWord(testNgram, 3);
						//comprobar si freq es mayor que frequencyOfNgram4th
						System.out.println(freq);
						if (freq>frequencyOfNgram4th){
							System.out.println("\t"+word +" seems to be a better word because its frequency is:"+freq);
							ArrayList<String> sug=suggestions.get(new Integer(ngramCount+3));
							if (sug==null) sug=new ArrayList<String>();
							if (!sug.contains(word)) sug.add(word);
							suggestions.put(new Integer(ngramCount+3), sug);
						}
					}
					
					System.out.println("Finding candidates for:"+splitNgram[4]);
					
					int frequencyOfNgram5th=c.getFrequencyOfNgram5thWord(ngram, 5)+c.getFrequencyOfNgram5thWord(ngram, 4)+c.getFrequencyOfNgram5thWord(ngram, 3);
					System.out.println(" Freq of '"+splitNgram[4]+"' in this context:"+frequencyOfNgram5th);
					
					Iterator<String> it5=fifthWordConfList.iterator();
					while(it5.hasNext()){
						String word=it5.next();
						String testNgram=splitNgram[0]+" "+splitNgram[1]+" "+splitNgram[2]+" "+splitNgram[3]+" "+word;
						System.out.print("\tTEST:"+testNgram+":");
						
						int freq=c.getFrequencyOfNgram5thWord(testNgram, 5)+c.getFrequencyOfNgram5thWord(testNgram, 4)+c.getFrequencyOfNgram5thWord(testNgram, 3);
						System.out.println(freq);
						if (freq>frequencyOfNgram5th){
							System.out.println("\t"+word +" seems to be a better word because its frequency is:"+freq);
							ArrayList<String> sug=suggestions.get(new Integer(ngramCount+4));
							if (sug==null) sug=new ArrayList<String>();
							if (!sug.contains(word)) sug.add(word);
							suggestions.put(new Integer(ngramCount+4), sug);
						}
						//comprobar si freq es mayor que frequencyOfNgram5th
						
					}	
					ngramCount++;
				}
				System.out.println("\n");
				for (int k=0;k<split.length;k++)
					System.out.print(k+":"+split[k]+" ");
				System.out.println();
				System.out.println("Suggestions:"+suggestions);
				suggestions=new HashMap<Integer,ArrayList<String>>();
				time_end = System.currentTimeMillis();

				System.out.println("\nThe task has taken "+ ( time_end - time_start ) +" milliseconds");

				
			}
			br.close();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} //fichero con textos de ejemplo. Por lineas.
		*/
	}
	
	
	public HashMap<Integer,ArrayList<String>> runCorrector(String text){
		try {
			//BufferedReader br = new BufferedReader(new FileReader(args[2]));
			Scanner s=new Scanner(text);
			HashMap<Integer,ArrayList<String>> suggestions=new HashMap<Integer,ArrayList<String>>();
			
		
			
			while(s.hasNext()){
				
				long time_start, time_end;
				time_start = System.currentTimeMillis();

				String line=s.nextLine(); //asumimos que está tokenizado...
				System.out.println("---------------------------------------------------");
				line=line.toLowerCase();
				System.out.println(line);
				
				
				ArrayList<String> fivegrams=new ArrayList<String>();
				String split[]=line.split(" ");
				
				//word0 word1 word2 word3 word4 word5 word6 word7...
				//HashMap<Integer,ArrayList<String>> suggestions=new HashMap<Integer,ArrayList<String>>();
				
				
				//dividirlo en ngrams de 5.
				for(int i=0;i<split.length-4;i++){
					String fivegram="";
					for(int j=i;j<5+i&&j<split.length;j++){
						fivegram+=split[j]+" ";
					}
					fivegram=fivegram.substring(0,fivegram.length()-1);
					if (!fivegrams.contains(fivegram)) fivegrams.add(fivegram);
				}
				
				System.out.println("f:"+fivegrams);
				
				Iterator<String> itfive=fivegrams.iterator();
				int ngramCount=0;
				
				
				//SENTENCE SCORE IN THE LANGUAGE MODEL
				List<String> list= Arrays.asList(line.trim().split("\\s+"));
				float logProb=lm.scoreSentence(list);
				System.out.println("Score of:"+line+": "+ logProb);
				
				
				while(itfive.hasNext()){
					String ngram=itfive.next();
					ArrayList<String> firstWordConfList=FirstWord(ngram);
					ArrayList<String> secondWordConfList=SecondWord(ngram);
					ArrayList<String> thirdWordConfList=ThirdWord(ngram);
					ArrayList<String> fourthWordConfList=FourthWord(ngram);
					ArrayList<String> fifthWordConfList=FifthWord(ngram);
					
					
					String splitNgram[]=ngram.split(" ");
					
					/*List<String> listNgram= Arrays.asList(ngram.trim().split("\\s+"));
					float logProb=lm.scoreSentence(listNgram);
					System.out.println("Score of:"+line+": "+ logProb);*/
					
					//System.out.println(ngram);
					System.out.println("Finding candidates for:"+splitNgram[0]);
					
					
					/*****************************For the first 5gram**********************/
					if (ngramCount==0){
						
						int frequencyOfNgram1st=getFrequencyOfNgram1stWord(ngram, 5)+getFrequencyOfNgram1stWord(ngram, 4);//+getFrequencyOfNgram1stWord(ngram, 3);
						//int frequencyOfNgram1st=this.getFrequencyOfNgram(ngram, 5,ngram.substring(0, 2))+getFrequencyOfNgram(ngram, 4,ngram.substring(0, 2));//+getFrequencyOfNgram1stWord(ngram, 3);
						System.out.println(" Freq of '"+splitNgram[0]+"' in this context:"+frequencyOfNgram1st);
						
						Iterator<String> it1=firstWordConfList.iterator();
						while(it1.hasNext()){
							String word=it1.next();
							String testNgram=word+" "+splitNgram[1]+" "+splitNgram[2]+" "+splitNgram[3]+" "+splitNgram[4];
							System.out.print("\tTEST:"+testNgram+":");
							
							int freq=getFrequencyOfNgram1stWord(testNgram, 5)+getFrequencyOfNgram1stWord(testNgram, 4);//+getFrequencyOfNgram1stWord(testNgram, 3);
							//int freq=this.getFrequencyOfNgram(testNgram, 5,testNgram.substring(0, 2))+getFrequencyOfNgram(testNgram, 4,testNgram.substring(0, 2));//+getFrequencyOfNgram1stWord(ngram, 3);
							System.out.println(freq);
							//comprobar si freq es mayor que frequencyOfNgram1st
							if (freq>frequencyOfNgram1st){
								System.out.println("\t"+word +" seems to be a better word because its frequency is:"+freq);
								
								//ComputeLogProbabilityOfTextStream a=new ComputeLogProbabilityOfTextStream();
								//edu.berkeley.nlp.lm.io.
								
								String[] sentenceSplit=line.split(" ");
								String sentenceTest="";
								for(int i=0;i<sentenceSplit.length;i++){
									if (i==ngramCount) sentenceTest+=word+" ";
									else sentenceTest+=sentenceSplit[i]+" ";
								}
								sentenceTest=sentenceTest.substring(0, sentenceTest.length()-1);

								List<String> testList= Arrays.asList(sentenceTest.trim().split("\\s+"));
								float logProbCand=lm.scoreSentence(testList);
								System.out.println("Score of candidate sentence:"+sentenceTest+": "+ logProbCand);
								
								if (logProbCand+5>logProb){ //threshold of 5
	
									if (grammarCheck(sentenceTest, ngramCount)){
										ArrayList<String> sug=suggestions.get(new Integer(ngramCount));
										if (sug==null) sug=new ArrayList<String>();
										if (!sug.contains(word)) sug.add(word);
										suggestions.put(new Integer(ngramCount), sug);
									}
								}
								
								
							}
						}

						System.out.println("Finding candidates for:"+splitNgram[1]);
						
						int frequencyOfNgram2nd=getFrequencyOfNgram2ndWord(ngram, 5)+getFrequencyOfNgram2ndWord(ngram, 4);//+getFrequencyOfNgram2ndWord(ngram, 3);
						//int frequencyOfNgram2nd=this.getFrequencyOfNgram(ngram, 5,ngram.substring(0, 2))+getFrequencyOfNgram(ngram, 4,ngram.substring(0, 2));//+getFrequencyOfNgram1stWord(ngram, 3);
						System.out.println(" Freq of '"+splitNgram[1]+"' in this context:"+frequencyOfNgram2nd);
						
						
						Iterator<String> it2=secondWordConfList.iterator();
						while(it2.hasNext()){
							String word=it2.next();
							String testNgram=split[0]+" "+word+" "+splitNgram[2]+" "+splitNgram[3]+" "+splitNgram[4];
							System.out.print("\tTEST:"+testNgram+":");
							
							int freq=getFrequencyOfNgram2ndWord(testNgram, 5)+getFrequencyOfNgram2ndWord(testNgram, 4);//+getFrequencyOfNgram2ndWord(testNgram, 3);
							//int freq=this.getFrequencyOfNgram(testNgram, 5,testNgram.substring(0, 2))+getFrequencyOfNgram(testNgram, 4,testNgram.substring(0, 2));//+getFrequencyOfNgram1stWord(ngram, 3);
							
							System.out.println(freq);
							//comprobar si freq es mayor que frequencyOfNgram2nd
							if (freq>frequencyOfNgram2nd){
								System.out.println("\t"+word +" seems to be a better word because its frequency is:"+freq);		
								
								String[] sentenceSplit=line.split(" ");
								String sentenceTest="";
								for(int i=0;i<sentenceSplit.length;i++){
									if (i==ngramCount+1) sentenceTest=word+" ";
									else sentenceTest+=sentenceSplit[i]+" ";
								}
								sentenceTest=sentenceTest.substring(0, sentenceTest.length()-1);
								
								List<String> testList= Arrays.asList(sentenceTest.trim().split("\\s+"));
								float logProbCand=lm.scoreSentence(testList);
								System.out.println("Score of candidate sentence:"+sentenceTest+": "+ logProbCand);
								
								if (logProbCand+10>logProb){
									if (grammarCheck(sentenceTest, ngramCount+1)){
										ArrayList<String> sug=suggestions.get(new Integer(ngramCount+1));
										if (sug==null) sug=new ArrayList<String>();
										if (!sug.contains(word)) sug.add(word);
										suggestions.put(new Integer(ngramCount+1), sug);
									}
								}
							}
							
						}
					}
					
					/**********************For all ngrams***************************/
						System.out.println("Finding candidates for:"+splitNgram[2]);
						
						int frequencyOfNgram3rd=getFrequencyOfNgram3rdWord(ngram, 5)+getFrequencyOfNgram3rdWord(ngram, 4);//+getFrequencyOfNgram3rdWord(ngram, 3);
						//int frequencyOfNgram3rd=this.getFrequencyOfNgram(ngram, 5,ngram.substring(0, 2))+getFrequencyOfNgram(ngram, 4,ngram.substring(0, 2));//+getFrequencyOfNgram1stWord(ngram, 3);
						System.out.println(" Freq of '"+splitNgram[2]+"' in this context:"+frequencyOfNgram3rd);
						
						Iterator<String> it3=thirdWordConfList.iterator();
						while(it3.hasNext()){
							String word=it3.next();
							String testNgram=splitNgram[0]+" "+splitNgram[1]+" "+word+" "+splitNgram[3]+" "+splitNgram[4];
							System.out.print("\tTEST:"+testNgram+":");
							
							int freq=getFrequencyOfNgram3rdWord(testNgram, 5)+getFrequencyOfNgram3rdWord(testNgram, 4);//+getFrequencyOfNgram3rdWord(testNgram, 3);
							//int freq=this.getFrequencyOfNgram(testNgram, 5,testNgram.substring(0, 2))+getFrequencyOfNgram(testNgram, 4,testNgram.substring(0, 2));//+getFrequencyOfNgram1stWord(ngram, 3);
							System.out.println(freq);
							if (freq>frequencyOfNgram3rd){
								System.out.println("\t"+word +" seems to be a better word because its frequency is:"+freq);
								
								String[] sentenceSplit=line.split(" ");
								String sentenceTest="";
								for(int i=0;i<sentenceSplit.length;i++){
									if (i==ngramCount+2) sentenceTest+=word+" ";
									else sentenceTest+=sentenceSplit[i]+" ";
								}
								sentenceTest=sentenceTest.substring(0, sentenceTest.length()-1);
								
								List<String> testList= Arrays.asList(sentenceTest.trim().split("\\s+"));
								float logProbCand=lm.scoreSentence(testList);
								System.out.println("Score of candidate sentence:"+sentenceTest+": "+ logProbCand);
								
								if ((logProbCand+10)>logProb){
									if (grammarCheck(sentenceTest, ngramCount+2)){
										ArrayList<String> sug=suggestions.get(new Integer(ngramCount+2));
										if (sug==null) sug=new ArrayList<String>();
										if (!sug.contains(word)) sug.add(word);
										suggestions.put(new Integer(ngramCount+2), sug);
									}
								}
							}
							//comprobar si freq es mayor que frequencyOfNgram3rd
						}
						
						/**********************For the last 5gram***************************/
						
					if (ngramCount==fivegrams.size()-1){
						System.out.println("Finding candidates for:"+splitNgram[3]);
						
						int frequencyOfNgram4th=getFrequencyOfNgram4thWord(ngram, 5)+getFrequencyOfNgram4thWord(ngram, 4);//+getFrequencyOfNgram4thWord(ngram, 3);
						//int frequencyOfNgram4th=this.getFrequencyOfNgram(ngram, 5,ngram.substring(0, 2))+getFrequencyOfNgram(ngram, 4,ngram.substring(0, 2));//+getFrequencyOfNgram1stWord(ngram, 3);
						System.out.println(" Freq of '"+splitNgram[3]+"' in this context:"+frequencyOfNgram4th);
						
						Iterator<String> it4=fourthWordConfList.iterator();
						while(it4.hasNext()){
							String word=it4.next();
							String testNgram=splitNgram[0]+" "+splitNgram[1]+" "+splitNgram[2]+" "+word+" "+splitNgram[4];
							System.out.print("\tTEST:"+testNgram+":");
							
							int freq=getFrequencyOfNgram4thWord(testNgram, 5)+getFrequencyOfNgram4thWord(testNgram, 4);//+getFrequencyOfNgram4thWord(testNgram, 3);
							//int freq=this.getFrequencyOfNgram(testNgram, 5,testNgram.substring(0, 2))+getFrequencyOfNgram(testNgram, 4,testNgram.substring(0, 2));//+getFrequencyOfNgram1stWord(ngram, 3);
							//comprobar si freq es mayor que frequencyOfNgram4th
							System.out.println(freq);
							if (freq>frequencyOfNgram4th){
								
								String[] sentenceSplit=line.split(" ");
								String sentenceTest="";
								for(int i=0;i<sentenceSplit.length;i++){
									if (i==ngramCount+3) sentenceTest+=word+" ";
									else sentenceTest+=sentenceSplit[i]+" ";
								}
								sentenceTest=sentenceTest.substring(0, sentenceTest.length()-1);
								
								List<String> testList= Arrays.asList(sentenceTest.trim().split("\\s+"));
								float logProbCand=lm.scoreSentence(testList);
								System.out.println("Score of candidate sentence:"+sentenceTest+": "+ logProbCand);
								
							
								if (logProbCand+10>logProb){
									if (grammarCheck(sentenceTest, ngramCount+3)){
										ArrayList<String> sug=suggestions.get(new Integer(ngramCount+3));
										if (sug==null) sug=new ArrayList<String>();
										if (!sug.contains(word)) sug.add(word);
										suggestions.put(new Integer(ngramCount+3), sug);
									}
								}
							}
						}
						
						System.out.println("Finding candidates for:"+splitNgram[4]);
						
						int frequencyOfNgram5th=getFrequencyOfNgram5thWord(ngram, 5)+getFrequencyOfNgram5thWord(ngram, 4);//+getFrequencyOfNgram5thWord(ngram, 3);
						//int frequencyOfNgram5th=this.getFrequencyOfNgram(ngram, 5,ngram.substring(0, 2))+getFrequencyOfNgram(ngram, 4,ngram.substring(0, 2));//+getFrequencyOfNgram1stWord(ngram, 3);
						System.out.println(" Freq of '"+splitNgram[4]+"' in this context:"+frequencyOfNgram5th);
						
						Iterator<String> it5=fifthWordConfList.iterator();
						while(it5.hasNext()){
							String word=it5.next();
							String testNgram=splitNgram[0]+" "+splitNgram[1]+" "+splitNgram[2]+" "+splitNgram[3]+" "+word;
							System.out.print("\tTEST:"+testNgram+":");
							
							int freq=getFrequencyOfNgram5thWord(testNgram, 5)+getFrequencyOfNgram5thWord(testNgram, 4);//+getFrequencyOfNgram5thWord(testNgram, 3);
							//int freq=this.getFrequencyOfNgram(testNgram, 5,testNgram.substring(0, 2))+getFrequencyOfNgram(testNgram, 4,testNgram.substring(0, 2));//+getFrequencyOfNgram1stWord(ngram, 3);
							System.out.println(freq);
							if (freq>frequencyOfNgram5th){
								
								String[] sentenceSplit=line.split(" ");
								String sentenceTest="";
								for(int i=0;i<sentenceSplit.length;i++){
									if (i==ngramCount+4) sentenceTest+=word+" ";
									else sentenceTest+=sentenceSplit[i]+" ";
								}
								sentenceTest=sentenceTest.substring(0, sentenceTest.length()-1);
								
								List<String> testList= Arrays.asList(sentenceTest.trim().split("\\s+"));
								float logProbCand=lm.scoreSentence(testList);
								System.out.println("Score of candidate sentence:"+sentenceTest+": "+ logProbCand);
								
								if (logProbCand+10>logProb){
									if (grammarCheck(sentenceTest, ngramCount+4)){
										ArrayList<String> sug=suggestions.get(new Integer(ngramCount+4));
										if (sug==null) sug=new ArrayList<String>();
										if (!sug.contains(word)) sug.add(word);
										suggestions.put(new Integer(ngramCount+4), sug);
									}
								}
							}
							//comprobar si freq es mayor que frequencyOfNgram5th
							
						}	
					}
					ngramCount++;
				}
				System.out.println("\n");
				for (int k=0;k<split.length;k++)
					System.out.print(k+":"+split[k]+" ");
				System.out.println();
				System.out.println("Suggestions:"+suggestions);
				suggestions=new HashMap<Integer,ArrayList<String>>();
				time_end = System.currentTimeMillis();

				System.out.println("\nThe task has taken "+ ( time_end - time_start ) +" milliseconds");
				return suggestions;

				
			}
			s.close();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} //fichero con textos de ejemplo. Por lineas.
		return null;
	}
	
	
	private boolean grammarCheck(String sentence, int index) {
		// TODO Auto-generated method stub
		
		try{
			
			String[] sentenceSplit=sentence.split(" ");
			
			String word="";
			
			String prevword="";
			
			if (sentenceSplit.length>index) word=sentenceSplit[index];
			if (sentenceSplit.length>index && (index-1)>0) prevword=sentenceSplit[index-1];
			
			String forms[] = ("<root> " + sentence).split(" ");
			SentenceData09 d = new SentenceData09();
			d.init(forms);
			d.plabels = new String[forms.length];
			d.pheads = new int[forms.length];
			d.pfeats = new String[forms.length];
			d.lemmas = new String[forms.length];
			d.plemmas = new String[forms.length];

			//d.forms =forms;
			d = tbparser.apply(d);
			
			String lemmaprev="";
			if (d.plemmas.length>index) lemmaprev=d.plemmas[index]; //in the dependency tree we have the root so index is the "PREVIOUS WORD"
			
			System.out.println(lemmaprev);
			System.out.println(word);
			
		
			
			
			
			if (lemmaprev.equals("haber") && word.endsWith("ndo")) return false;
			if (lemmaprev.equals("haber") && !word.endsWith("do")) return false;
			if (lemmaprev.equals("estar") && !word.endsWith("ndo")) return false;
			
			String featsprev="";
			String featsword="";
			int headprev=-1000;
			if (d.pfeats.length>index) featsprev=d.pfeats[index];
			if (d.pfeats.length>index+1) featsword=d.pfeats[index+1];
			if (d.pheads.length>index) headprev=d.pheads[index]; //dependency head of the word
			
			System.out.println(featsprev);
			System.out.println(featsword);
			System.out.println(headprev);
			
			
			if (featsprev.contains("num=p") && featsword.contains("num=s") && headprev==(index+1)) return false;
			if (featsprev.contains("num=p") && featsword.contains("num=s")) return false;
			//if (featsprev.contains("num=s") && featsword.contains("num=p") && headprev==(index+1)) return false;
			if (featsprev.contains("num=s") && featsword.contains("num=p")) return false;
			
			if (featsprev.contains("num=s") && featsword.contains("num=p") && headprev==(index+1)) return false;
			System.out.println(prevword);
			
			if (word.endsWith("s") && !prevword.endsWith("s")) return false;
			
			if (word.endsWith("me") || word.endsWith("le") || word.endsWith("se") || word.endsWith("te")){
				for (int i=0;i<d.pheads.length;i++){
					if (d.pheads[i]==(index+1)){
						if (forms[i].equals("me") || forms[i].equals("le") || forms[i].equals("se") || forms[i].equals("te")) return false;
					}
				}
			}
			
			/*if (word.endsWith("se")){
				for (int i=0;i<d.pheads.length;i++){
					if (d.pheads[i]==(index+1)){
						if (forms[i].equals("se")) return false;
					}
				}
			}
			
			if (word.endsWith("le")){
				for (int i=0;i<d.pheads.length;i++){
					if (d.pheads[i]==(index+1)){
						if (forms[i].equals("le")) return false;
					}
				}
			}*/
			
		
		}catch(Exception e){
			return true;
		}
		return true;
	}

	/**
	 * @param files
	 * @param lm
	 * @throws IOException
	 * @throws FileNotFoundException
	 */
	private double logProb(List<String> files) throws IOException, FileNotFoundException {
		double prob = 0.0;
		for (String file : files) {
			final InputStream is = (file.equals("-")) ? System.in : (file.endsWith(".gz") ? new GZIPInputStream(new FileInputStream(file))
				: new FileInputStream(file));
			BufferedReader reader = new BufferedReader(new InputStreamReader(new BufferedInputStream(is)));
			for (String line : Iterators.able(IOUtils.lineIterator(reader))) {
				List<String> words = Arrays.asList(line.trim().split("\\s+"));
				prob += lm.getLogProb(words);
			}
		}
		return prob;
	}

	/**
	 * @param isGoogleBinary
	 * @param vocabFile
	 * @param binaryFile
	 * @return
	 */
	private NgramLanguageModel<String> readBinary(boolean isGoogleBinary, String vocabFile, String binaryFile) {
		NgramLanguageModel<String> lm = null;
		if (isGoogleBinary) {
			
			lm = LmReaders.readGoogleLmBinary(binaryFile, vocabFile);
		} else {
			lm = LmReaders.readLmBinary(binaryFile);
		}
		return lm;
	}
	
	
	

}

